@@ -25,14 +25,9 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+#include <time.h>
 
-#ifdef ENABLE_GNOME
-#include <libgnomeui/libgnomeui.h>
-#include <libgnome/gnome-program.h>
-#include <libgnome/gnome-help.h>
-#else
 #include <stdlib.h>
-#endif
 
 #include <string.h>
 #include "gst-tool.h"
@@ -46,6 +41,11 @@ enum {
 	PLATFORM_LIST_COL_LAST
 };
 
+enum {
+	OPERATION_COMMIT,
+	OPERATION_UPDATE
+};
+
 static void  gst_tool_class_init   (GstToolClass *class);
 static void  gst_tool_init         (GstTool      *tool);
 static void  gst_tool_finalize     (GObject      *object);
@@ -118,42 +118,42 @@ report_window_close_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
 	return TRUE;
 }
 
-static GladeXML*
-gst_tool_load_glade_common (GstTool *tool, const gchar *widget)
+static GtkBuilder *
+gst_tool_load_common_ui (GstTool *tool)
 {
-	GladeXML *xml;
-	
+	GtkBuilder *builder;
+
 	g_return_val_if_fail (tool != NULL, NULL);
 	g_return_val_if_fail (GST_IS_TOOL (tool), NULL);
-	g_return_val_if_fail (tool->glade_common_path != NULL, NULL);
+	g_return_val_if_fail (tool->common_ui_path != NULL, NULL);
 
-	xml = glade_xml_new (tool->glade_common_path, widget, NULL);
+	builder = gtk_builder_new ();
 
-	if (!xml) {
-		g_error ("Could not load %s\n", tool->glade_common_path);
+	if (!gtk_builder_add_from_file (builder, tool->common_ui_path, NULL)) {
+		g_error ("Could not load %s\n", tool->common_ui_path);
 	}
 
-	return xml;
+	return builder;
 }
 
 static void
 gst_tool_init (GstTool *tool)
 {
-	GdkPixbuf *pixbuf;
-	GladeXML  *xml;
+	GdkPixbuf  *pixbuf;
+	GtkBuilder *builder;
 
 	tool->icon_theme = gtk_icon_theme_get_default ();
-	tool->glade_common_path  = INTERFACES_DIR "/common.glade";
+	tool->common_ui_path  = INTERFACES_DIR "/common.ui";
 
 	tool->session = oobs_session_get ();
 	tool->gconf_client = gconf_client_get_default ();
 
-	xml = gst_tool_load_glade_common (tool, "report_window");
+	builder = gst_tool_load_common_ui (tool);
 
-	tool->report_window = glade_xml_get_widget (xml, "report_window");
-	tool->report_label = glade_xml_get_widget (xml, "report_label");
-	tool->report_progress = glade_xml_get_widget (xml, "report_progress");
-	tool->report_pixmap = glade_xml_get_widget (xml, "report_pixmap");
+	tool->report_window = GTK_WIDGET (gtk_builder_get_object (builder, "report_window"));
+	tool->report_label = GTK_WIDGET (gtk_builder_get_object (builder, "report_label"));
+	tool->report_progress = GTK_WIDGET (gtk_builder_get_object (builder, "report_progress"));
+	tool->report_pixmap = GTK_WIDGET (gtk_builder_get_object (builder, "report_pixmap"));
 	g_signal_connect (G_OBJECT (tool->report_window), "delete_event",
 			  G_CALLBACK (report_window_close_cb), tool);
 
@@ -161,23 +161,35 @@ gst_tool_init (GstTool *tool)
 	gtk_image_set_from_pixbuf (GTK_IMAGE (tool->report_pixmap), pixbuf);
 
 	if (pixbuf)
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 
-	g_object_unref (xml);
+	tool->objects = g_ptr_array_new ();
+
+	g_object_unref (builder);
 }
 
 static void
-show_access_denied_dialog (GstTool *tool)
+show_access_denied_dialog (GstTool *tool,
+			   gint     operation)
 {
 	GtkWidget *dialog;
+	const gchar *primary_text, *secondary_text;
 
-	dialog = gtk_message_dialog_new (NULL,
+	if (operation == OPERATION_UPDATE) {
+		primary_text = N_("The configuration could not be loaded");
+		secondary_text = N_("You are not allowed to access the system configuration.");
+	} else {
+		primary_text = N_("The configuration could not be saved");
+		secondary_text = N_("You are not allowed to modify the system configuration.");
+	}
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
 					 GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_CLOSE,
-					 _("The configuration could not be loaded"));
+					 _(primary_text));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("You are not allowed to access the system configuration."));
+						  _(secondary_text));
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
@@ -206,7 +218,7 @@ gst_tool_constructor (GType                  type,
 		gtk_window_set_default_icon_name (tool->icon);
 
 	if (tool->name) {
-		tool->glade_path = g_strdup_printf (INTERFACES_DIR "/%s.glade", tool->name);
+		tool->ui_path = g_strdup_printf (INTERFACES_DIR "/%s.ui", tool->name);
 
 		widget_name = g_strdup_printf ("%s_admin", tool->name);
 		tool->main_dialog = gst_dialog_new (tool, widget_name, tool->title);
@@ -220,18 +232,18 @@ gst_tool_constructor (GType                  type,
 		dialog = gst_platform_dialog_new (tool->session);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
-
-		gst_tool_update_config (tool);
 		break;
 	case OOBS_RESULT_ACCESS_DENIED:
 	case OOBS_RESULT_ERROR:
-		show_access_denied_dialog (tool);
+		show_access_denied_dialog (tool, OPERATION_UPDATE);
 		exit (-1);
 		break;
 	default:
 		break;
 	}
 
+	gst_tool_update_async (tool);
+
 	return object;
 }
 
@@ -264,7 +276,7 @@ gst_tool_finalize (GObject *object)
 	g_free (tool->name);
 	g_free (tool->title);
 	g_free (tool->icon);
-	g_free (tool->glade_path);
+	g_free (tool->ui_path);
 
 	if (tool->session)
 		g_object_unref (tool->session);
@@ -278,6 +290,8 @@ gst_tool_finalize (GObject *object)
 	if (tool->gconf_client)
 		g_object_unref (tool->gconf_client);
 
+	g_ptr_array_free (tool->objects, FALSE);
+
 	(* G_OBJECT_CLASS (gst_tool_parent_class)->finalize) (object);
 }
 
@@ -292,6 +306,8 @@ gst_tool_impl_close (GstTool *tool)
 
 	/* process pending async requests */
 	oobs_session_process_requests (tool->session);
+
+
 	g_object_unref (tool);
 	gtk_main_quit ();
 }
@@ -323,19 +339,9 @@ gst_tool_close (GstTool *tool)
 		(* GST_TOOL_GET_CLASS (tool)->close) (tool);
 }
 
-gboolean
-gst_tool_is_authenticated (GstTool *tool)
-{
-	/* FIXME */
-	return TRUE;
-}
-
 void
 gst_init_tool (const gchar *app_name, int argc, char *argv [], GOptionEntry *entries)
 {
-#ifdef ENABLE_GNOME
-	GnomeProgram   *program;
-#endif
 	GOptionContext *context;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -350,16 +356,7 @@ gst_init_tool (const gchar *app_name, int argc, char *argv [], GOptionEntry *ent
 		g_option_context_free (context);
 	}
 
-#ifdef ENABLE_GNOME
- 	program = gnome_program_init (app_name, VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_APP_DATADIR, DATADIR,
-				      GNOME_PARAM_HUMAN_READABLE_NAME,
-				      _("GNOME System Tools"),
-				      NULL);
-#else
-	gtk_init (&argc, &argv);	
-#endif
+	gtk_init (&argc, &argv);
 }
 
 void
@@ -388,12 +385,12 @@ gst_tool_show_help (GstTool *tool, gchar *section)
 				       lang,
 				       help_file_xml,
 				       NULL);
-					
+
 		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
                     break;
 		}
 	}
-	
+
 	if (section) {
 		command = g_strconcat ("gnome-help ghelp://", uri, "?", section, NULL);
 	} else {
@@ -484,6 +481,16 @@ on_commit_finalized (OobsObject *object,
 
 	gst_tool_hide_report_window (user_data->tool);
 
+	switch (result) {
+	case OOBS_RESULT_ACCESS_DENIED:
+		show_access_denied_dialog (user_data->tool, OPERATION_COMMIT);
+		break;
+	case OOBS_RESULT_OK:
+	default:
+		/* FIXME: handle other errors */
+		break;
+	}
+
 	if (user_data->func)
 		(* user_data->func) (object, result, user_data->data);
 
@@ -504,12 +511,118 @@ gst_tool_commit_async (GstTool             *tool,
 	user_data->func = func;
 	user_data->data = data;
 
-	gst_tool_show_report_window (tool, message);
+	tool->last_commit_time = time (NULL);
+
+	if (message)
+		gst_tool_show_report_window (tool, message);
+
 	oobs_object_commit_async (object, on_commit_finalized, user_data);
 }
 
+static void
+update_async_func (OobsObject *object,
+		   OobsResult  result,
+		   gpointer    data)
+{
+	GstTool *tool = GST_TOOL (data);
+	gst_dialog_thaw (tool->main_dialog);
+
+	if (gst_dialog_get_freeze_level (tool->main_dialog) == 0) {
+		/* everything is now updated */
+		gst_tool_update_config (tool);
+		gst_tool_update_gui (tool);
+	}
+}
+
+void
+gst_tool_update_async (GstTool *tool)
+{
+	gint i;
+
+	g_return_if_fail (GST_IS_TOOL (tool));
+
+	for (i = 0; i < tool->objects->len; i++) {
+		OobsObject *object = g_ptr_array_index (tool->objects, i);
+
+		gst_dialog_freeze (tool->main_dialog);
+		oobs_object_update_async (object,
+					  update_async_func,
+					  tool);
+	}
+}
+
 GtkIconTheme*
 gst_tool_get_icon_theme (GstTool *tool)
 {
 	return tool->icon_theme;
 }
+
+static void
+configuration_object_changed (OobsObject *object,
+			      GstTool    *tool)
+{
+	gboolean do_update = TRUE;
+	time_t current_time;
+
+	current_time = time (NULL);
+
+	/* If we get ::changed shortly after having committed,
+	 * there's a good chance that the tool has been the
+	 * origin of the change. Of course there could be the
+	 * possibility that this isn't true, but will happen
+	 * quite rarely.
+	 */
+	if (current_time - tool->last_commit_time <= 2)
+		return;
+
+	if (gst_dialog_get_editing (tool->main_dialog)) {
+		GtkWidget *parent, *dialog;
+		gint response;
+
+		if (tool->configuration_changed_dialog)
+			return;
+
+		parent = gst_dialog_get_topmost_edit_dialog (tool->main_dialog);
+		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_QUESTION,
+						 GTK_BUTTONS_YES_NO,
+						 _("The system configuration has potentially changed."));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("Update content? This will lose any modification in course."));
+		tool->configuration_changed_dialog = dialog;
+
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		do_update = (response == GTK_RESPONSE_YES);
+
+		gtk_widget_destroy (dialog);
+		tool->configuration_changed_dialog = NULL;
+	}
+
+	if (do_update) {
+		gst_dialog_stop_editing (tool->main_dialog);
+		gst_tool_update_async (tool);
+	}
+}
+
+static void
+configuration_object_committed (OobsObject *object,
+				GstTool    *tool)
+{
+	tool->last_commit_time = time (NULL);
+}
+
+void
+gst_tool_add_configuration_object (GstTool    *tool,
+				   OobsObject *object)
+{
+	g_return_if_fail (GST_IS_TOOL (tool));
+	g_return_if_fail (OOBS_IS_OBJECT (object));
+
+	g_ptr_array_add (tool->objects, object);
+
+	g_signal_connect (object, "changed",
+			  G_CALLBACK (configuration_object_changed), tool);
+	g_signal_connect (object, "committed",
+			  G_CALLBACK (configuration_object_committed), tool);
+}
