@@ -69,6 +69,7 @@ add_user_columns (GtkTreeView *treeview)
 							   "text", COL_USER_LOGIN,
 							   "sensitive", COL_USER_SENSITIVE,
 							    NULL);
+	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_insert_column (treeview, column, -1);
 
 	/* Home directory */
@@ -78,6 +79,7 @@ add_user_columns (GtkTreeView *treeview)
 							   "text", COL_USER_HOME,
 							   "sensitive", COL_USER_SENSITIVE,
 							   NULL);
+	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_insert_column (treeview, column, -1);
 }
 
@@ -204,3 +206,13 @@ users_table_add_user (OobsUser *user, OobsListIter *list_iter)
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	users_table_set_user (user, list_iter, &iter);
 }
+
+void
+users_table_clear (void)
+{
+        GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
+        GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
+        GtkTreeModel *model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+
+        gtk_list_store_clear (GTK_LIST_STORE (model));
+}
